/*
*   AbstractParamMethod - A partial implementation of an object that contains a method used to calculate an AutoParam.
*
*   Copyright (C) 1998-2012 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.ParamTools;

import javax.measure.quantity.Quantity;


/**
*  <p>  The base class for various parameter method objects which contain a 
*       method for calculating a given parameter's value from a given list
*       of input parameter values.
*  </p>
*
*  <p>  Modified by:  Joseph A. Huwaldt  </p>
*
*  @author   Joseph A. Huwaldt   Date: February 1, 1998
*  @version  September 15, 2012
**/
public abstract class AbstractParamMethod<Q extends Quantity> implements ParamMethod<Q>, java.io.Serializable {

	private static final long serialVersionUID = -8376041720168366632L;
	/**
	*  Array to hold labels of input parameters.  Subclasses must
	*  create and fill in this array.
	**/
	protected String[] _inputLabels;

	/**
	*  Do not allow users of this class to use the default constructor.
	**/
	protected AbstractParamMethod() { }

	/**
	*  Returns true if the inputs for this parameter are appropriate
	*  for this method.  Normally, "valid" means that the values for the input
	*  parameters are within a certain range.
	*  The default implementation always returns true.
	**/
    @Override
	public boolean validInputs(ParamDatabase pdb, String[] inputs) {
		return true;
	}
	
	/**
	*  Returns the number of inputs required by this method (the
	*  number of input parameter labels passed to the constructor when
	*  this object was instantiated).
	*
	*  @return  Returns the number of input values required by this method.
	**/
    @Override
	public int getNumInputs() {
		if ( _inputLabels != null )
			return _inputLabels.length;
		
		return 0;
	}

	/**
	*  Returns the label of the input parameter specified by "index".
	*
	*  @param   index  Index into the input label array for the label to be returned.
	*  @return  Returns the label for the input parameter specified.
	*  @exception  ArrayIndexOutOfBoundsException  Thrown if index is < 0 or greater
	*              than the number of parameters available.
	**/
    @Override
	public String getInputLabel( int index ) throws ArrayIndexOutOfBoundsException {
		return _inputLabels[index];
	}

	/**
	*  Returns the array of all the input parameter labels.
	*
	*  @return  The array of all input parameter label strings is returned.
	**/
    @Override
	public String[] getAllLabels() {
		return _inputLabels;
	}

}


