/*
*   AdvParamInput - Object used for entering AdvParam values using a text field.
*
*   Copyright (C) 1998-2012 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.ParamTools;

import java.awt.Component;
import java.awt.Insets;
import java.awt.Font;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.net.URL;
import java.net.MalformedURLException;
import java.text.NumberFormat;
import java.text.DecimalFormat;

import javax.swing.*;
import javax.measure.unit.Unit;
import javax.measure.quantity.Quantity;

import jahuwaldt.util.HelpSystem;
import jahuwaldt.swing.DecimalField;
import jahuwaldt.js.param.Parameter;


/**
*  <p>  Defines an Swing JPanel object that is used for entering
*       {@link AdvParam} type parameter values (using a text field).
*       Provides a GUI user interface to advanced parameter type
*       objects.  </p>
*
*  <p>  Modified by:  Joseph A. Huwaldt    </p>
*
*  @author  Joseph A. Huwaldt   Date: July 14, 1999
*  @version September 15, 2012
**/
public class AdvParamInput<Q extends Quantity> extends ParamInput<Q> {

	private static final long serialVersionUID = 5290575010422455062L;
	/**
	*  Panel for special controls at bottom of this panel.
	**/
	protected JPanel _bottomPanel;

	//-----------------------------------------------------------------------------
	/**
	*  Don't allow the user to access the default constructor.
	**/
	protected AdvParamInput() { }

	/**
	*  Constructs an AdvParamInput class that accepts an AdvParam for input.
	*
	*  @param  param The AdvParam object that this input field should use.
	*  @throws MalformedURLException if either the path to the symbol or help icon image
	*          files could not be combined with the application location and turned into a URL.
	**/
	public AdvParamInput( AdvParam<Q> param ) throws MalformedURLException {
		super(new BorderLayout());
		
		_theParam = param;
		
		//	Create text input field for this parameter.
		_nf = param.getNumberFormat();
		if (_nf == null)
			_nf = (DecimalFormat)NumberFormat.getInstance();

		if ( param.isDefined() )
			_theTField = new DecimalField( param.getValue(), 10, _nf );
		else
			_theTField = new DecimalField( 10, _nf );
			
		_tfActionListener = new TextFieldListener();
		_tfFocusListener = new TextFieldFocusL();
		_theTField.addActionListener( _tfActionListener );
		_theTField.addFocusListener( _tfFocusListener );
		add( _theTField, BorderLayout.CENTER );
		
		
		//	Create the label for this parameter.
		URL symbolURL = param.getSymbolURL();
		JLabel pLabel;
		if ( symbolURL == null ) {
			//	No graphic symbol, use text label.
			pLabel = new JLabel( param.getLabel() + " =", JLabel.RIGHT );
			pLabel.setFont(new Font("Serif", Font.PLAIN, 12));
			
		} else {
			Icon icon = new ImageIcon(symbolURL);
			
			//	Graphic symbol provided, so use that.
			pLabel = new JLabel( icon, JLabel.RIGHT );
		}
		
		pLabel.setLabelFor(_theTField);
		pLabel.setToolTipText(param.getDescription());
		add( pLabel, BorderLayout.WEST );
		
		
		//	Create the units label or choice menu for this parameter.
		if ( param._availUnits != null && param._availUnits.length > 1 ) {
		
			//	Many units are available.
			JComboBox unitList = new JComboBox(param.getAvailUnits());
			unitList.addActionListener(new UnitMenuListener());
			unitList.setToolTipText( RESOURCES.getString("availUnitsToolTip") );
			
			//	Select the current units.
			Unit<Q> currentUnit = param.getUnit();
			unitList.setSelectedItem( currentUnit );
			add( unitList, BorderLayout.EAST );
			
		} else {
			//	Only 1 unit available.
			String unitStr = " ";
			Unit<Q> theUnits = param.getUnit();
			if ( theUnits != null ) {
				unitStr += theUnits.toString();
			}
			unitStr += " ";
			JLabel theLabel = new JLabel( unitStr, JLabel.LEFT );
			add( theLabel, BorderLayout.EAST );
		}
		
		
		//	Add a panel to the bottom for special controls.
		_bottomPanel = new JPanel();
		add( _bottomPanel, BorderLayout.SOUTH );
		
		//	Add a help button to the bottom panel (if help is available).
		if ( param.isHelpAvail() ) {
		
			//	If help button images exist, create an ImageButton.
			URL helpIconURL = param.getHelpImageURL();
			JButton helpButton;
			
			if ( helpIconURL != null ) {
				helpButton = new JButton( new ImageIcon(helpIconURL) );
				helpButton.setBorderPainted(false);
			} else
				helpButton = new JButton( RESOURCES.getString("helpBtnText") );
			
			helpButton.setAlignmentX( JToggleButton.CENTER_ALIGNMENT );
			helpButton.setAlignmentY( JToggleButton.CENTER_ALIGNMENT );
			helpButton.setMargin( new Insets( 0, 0, 0, 0 ) );
			helpButton.addActionListener(new ButtonL());
			helpButton.setToolTipText( RESOURCES.getString("helpBtnToolTip") );
			_bottomPanel.add( helpButton );
			
		}
	}

	//-----------------------------------------------------------------------------
	/**
	*  Reads the value input in the text field and puts it into
	*  the corresponding parameter.  If the text edit field is empty,
	*  the parameter is made undefined.
	**/
    @Override
	public void getInput() {
		double newValue = _theTField.getValue();
		
		if (Double.isNaN(newValue))
			_theParam.makeUndefined();

		else {
			//	If the value has changed, reset the parameter.
			double dValue = newValue;
			if ( dValue != _oldValue ) {
				_theParam.setValue( dValue );
				
				//	Verify that the parameter the user has input is valid.
				ParamVerifier verifier = ((AdvParam<Q>)_theParam).getVerifier();
				if ( verifier != null ) {
					Parameter<Q> prevValue = _theParam.get();
					Component root = SwingUtilities.getRoot((Component)this);
					@SuppressWarnings("unchecked")
					Parameter<Q> value = (Parameter<Q>) verifier.check( root, prevValue );
					
					//	Call parameter verification method.
					if ( !prevValue.equals(value) ) {
						_theParam.set( value );
						
						//	Verification method changed value of param.
						resetText();
					}
				}
			}
		}
		
	}

	/**
	*  Defines a button listener for our help button.
	**/
	private class ButtonL implements ActionListener {

        @Override
		public  void actionPerformed( ActionEvent evt ) {
			
			//	Extract the help information from the parameter.
			HelpSystem help = ((AdvParam<Q>)_theParam).getHelpSystem();
			
			//	Show the help information for this parameter.
			if ( help != null )
				help.showHelp();
		}

	}
	
	
}


