/*
*   AutoParamInput - Object used for entering AutoParam values using a text field.
*
*   Copyright (C) 1998-2012 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.ParamTools;

import java.awt.Font;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusAdapter;
import java.text.ParseException;
import java.util.Observer;
import java.util.Observable;
import java.util.ArrayList;
import java.util.List;
import java.net.MalformedURLException;

import javax.swing.*;
import javax.swing.undo.UndoableEdit;
import javax.swing.undo.AbstractUndoableEdit;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.CannotRedoException;
import javax.swing.event.UndoableEditListener;
import javax.swing.event.UndoableEditEvent;
import javax.measure.quantity.Quantity;


/**
*  <p>  Defines a panel object that is used for entering
*       parameter values (using a text field).  This differs
*       from "ParamInput" objects in that these objects
*       will automatically call update methods for AutoParam
*       parameters.
*  </p>
*
*  <p>  Modified by:  Joseph A. Huwaldt   </p>
*
*  @author  Joseph A. Huwaldt   Date: January 13, 1998
*  @version September 15, 2012
**/
public class AutoParamInput<Q extends Quantity> extends AdvParamInput<Q> implements Observer {

	private static final long serialVersionUID = 6410350548798767148L;

	//	The manually input check box.
	private JCheckBox _manInputCB;
	
	//	A list of edit listeners.
	private final List<UndoableEditListener> _editListeners = new ArrayList<UndoableEditListener>();
	
	
	/**
	*  Constructs an AutoParamInput object using the given parameter.
	*
	*  @param  param The parameter object that this input field should use.
	*  @throws MalformedURLException if either the path to the symbol or help icon image
	*          files could not be combined with the application location and turned into a URL.
	**/
	public AutoParamInput( AutoParam<Q> param ) throws MalformedURLException {
		super( param );

		// Add a new focus and action listener to this ParamInput's text field.
		_theTField.removeFocusListener(_tfFocusListener);
		_theTField.removeActionListener(_tfActionListener);
		
		_tfFocusListener = new TextFieldFocusL2();
		_tfActionListener = new TextFieldListener2();
		
		_theTField.addFocusListener( _tfFocusListener );
		_theTField.addActionListener( _tfActionListener );

		//	Add a check box item that indicates if the object is manually input.
		_manInputCB = new JCheckBox( RESOURCES.getString("manBtnText"), param.isManuallyInput() );
		_manInputCB.addActionListener(new CheckboxListener());
		_manInputCB.setFont( new Font( "SansSerif", Font.PLAIN, 9 ) );
		_manInputCB.setToolTipText( RESOURCES.getString("manBtnToolTip") );
		_bottomPanel.add( _manInputCB );
		
		// If the parameter can only be manually input, disable the manual input check box.
		if (param.isManualOnly())
			_manInputCB.setEnabled(false);
		
		//	Make this text parameter input object an observer (child) of the parameter.
		param.addObserver(this);
		
		//	Record current value of this parameter.
		this.setOldValue();
	}

	//-----------------------------------------------------------------------------
	/**
	*  Reads the value input in the text field and puts it into
	*  the corresponding parameter.  If the TextEdit field is empty,
	*  the parameter is made undefined.
	**/
    @Override
	public void getInput() {
		
		//	Was a blank entered?
		if ( _theParam.isDefined() ) {
			String s = _theTField.getText();
			
			if ( s == null || s.equals( "" ) ) {
				//	Make the parameter undefined, and manually input.
				((AutoParam<Q>) _theParam).makeManuallyInput();
				_theParam.makeUndefined();
				return;
			}
		}
		
		super.getInput();
	}

	/**
	*  Called automatically by the Observable notification system when
	*  the parameter this object is dependent on has changed.
	*
	*  @param  o    The object being observed (this object)
	*  @param  arg  The parameter that this one depends on that has changed.
	**/
    @Override
	public void update( Observable o, Object arg ) {
		AutoParam<?> parentParam = (AutoParam<?>) arg;
		
		// Change the text display.
		resetText();
		
		// Make sure the manually input check box is up to date.
		_manInputCB.setSelected( parentParam.isManuallyInput() );
		
	}

	/**
	*  Add a listener to be notified each time an edit is made on the user 
	*  input in this panel.
	**/
	public synchronized void addUndoableEditListener(UndoableEditListener l) {
		_editListeners.add(l);
	}

	/**
	*  Remove a listener from this manager.
	**/
	public synchronized void removeUndoableEditListener(UndoableEditListener l) {
		_editListeners.remove(l);
	}

	/**
	*  Notifies all edit listeners that an edit has occurred.
	**/
	protected void notifyEditListeners(UndoableEdit e) {
		UndoableEditEvent ev = new UndoableEditEvent(this, e);
		for (UndoableEditListener listener : _editListeners)
			listener.undoableEditHappened(ev);
	}
	
	/**
	*  Defines a listener for our check box.
	**/
	private class CheckboxListener implements ActionListener {

        @Override
		public  void actionPerformed( ActionEvent evt ) {
			JCheckBox theCB = (JCheckBox)evt.getSource();
			AutoParam<Q> theAutoParam = (AutoParam<Q>)_theParam;
			
			// Record the effect for undo/redo.
			UndoableEdit edit = new AutoParamEdit(theAutoParam, theAutoParam.getValue(), theCB.isSelected(),
								RESOURCES.getString("changePrefix") + " "
								+ theAutoParam.getDescription());
			
			if ( !theCB.isSelected() ) {
				try {
				
					//	The user want's this param to be auto calculated.
					theAutoParam.makeUndefined();
					theAutoParam.makeAutoAndUpdate();
					
				} catch (Exception e) {
					// Shouldn't happen.
					e.printStackTrace();
				}
			} else {
				//	The user want's this to be manually input.
				theAutoParam.makeManuallyInput();
			}
			
			// Notify the undo/redo listeners
			notifyEditListeners(edit);
		}
	}
	
	
	/**
	*  Handle an event that indicates that the user may have changed
	*  the value in the text field of this parameter manually.
	**/
	private void handleInput() {
		double prevValue=0, newValue=0;
			
		try {
			// Round off the old value to the current display format.
			String text = _nf.format(_theParam.getValue());
			prevValue = _nf.parse(text).doubleValue();
			
			this.getInput();
			
			// Round off the new value to the current display format.
			text = _nf.format(_theParam.getValue());
			newValue = _nf.parse(text).doubleValue();

		} catch (ParseException err) {
			// Shouldn't be possible.
			err.printStackTrace();
		}
			
		//	Has the value been changed by the user (manually input)?
		if ( !Double.isNaN( newValue ) && newValue != prevValue ) {
			//	A new value was input.
			AutoParam<Q> theAutoParam = (AutoParam<Q>)_theParam;
			
			try {
				// Record the effect for undo/redo.
				UndoableEdit edit = new AutoParamEdit(theAutoParam, newValue, true,
									RESOURCES.getString("changePrefix") + " "
									+ theAutoParam.getDescription());
				
				_oldValue = newValue;
				
				//	Tell this parameter that it has been manually input.
				theAutoParam.makeManuallyInput();
				
				//	Notify parameters that are dependent on this one that they need to update.
				theAutoParam.updateChildren();
				
				// Notify the undo/redo listeners
				notifyEditListeners(edit);
				
			} catch (Exception err) {
				err.printStackTrace();
				JOptionPane.showMessageDialog ( this,
							RESOURCES.getString("badValueErr").replace("<NAME/>",err.getMessage()),
							RESOURCES.getString("updateErrTitle"),
							JOptionPane.ERROR_MESSAGE );
				theAutoParam.makeUndefined();
				theAutoParam.makeAuto();
			}
		}
	}
	
	/**
	*  Class used to record the changes made to the AutoParamInput state.
	**/
	@SuppressWarnings("serial")
	private class AutoParamEdit extends AbstractUndoableEdit {
		
		private AutoParam<Q> _theParam;
		private double _origValue, _newValue;
		private String _presName;
		private boolean _wasManuallyInput, _newManualInputFlag;
		
		public AutoParamEdit(AutoParam<Q> theParam, double newValue, boolean newManualInputFlag, String presentationName) {
			_theParam = theParam;
			_origValue = theParam.getValue();
			_newValue = newValue;
			_presName = presentationName;
			_wasManuallyInput = theParam.isManuallyInput();
			_newManualInputFlag = newManualInputFlag;
		}
		
        @Override
		public void undo() throws CannotUndoException {
			super.undo();
			if (_wasManuallyInput) {
				_theParam.makeManuallyInput();
				setParamValue(_origValue);
			} else  {
				_theParam.makeUndefined();
				_theParam.makeAutoAndUpdate();
			}
			resetText();
		}
		
        @Override
		public void redo() throws CannotRedoException {
			super.redo();
			if (_newManualInputFlag) {
				_theParam.makeManuallyInput();
				setParamValue(_newValue);
			} else {
				_theParam.makeUndefined();
				_theParam.makeAutoAndUpdate();
			}
			resetText();
		}
		
		//	Set the parameter value and update it's children.
		private void setParamValue(double newValue) {
			if (Double.isNaN(newValue))
				_theParam.makeUndefined();
			else {
				_theParam.setValue( newValue );
			}
			_theParam.updateChildren();
		}

        @Override
		public String getPresentationName() {
			return _presName;
		}
	}


	private boolean _keyPressed=false;
	/**
	*  Defines a listener for our text field that is called whenever the user
	*  types in a new number and presses enter.
	**/
	private class TextFieldListener2 implements ActionListener {
		
        @Override
		public void actionPerformed( ActionEvent e ) {
			_keyPressed = true;
			AutoParamInput.this.handleInput();
			_keyPressed = false;
		}
	}
	
	
	/**
	*  Defines a listener for our text field that is called whenever the
	*  focus changes off a particular text field.
	**/
	private class TextFieldFocusL2 extends FocusAdapter {

        @Override
		public void focusLost( FocusEvent e ) {
			if (!_keyPressed)
				AutoParamInput.this.handleInput();
			_keyPressed = false;
		}

	}

}


