/*
 *   ParamDBInputPanel - Panel that neatly displays ParamInput boxes.
 *
 *   Copyright (C) 1998-2015 by Joseph A. Huwaldt
 *   All rights reserved.
 *   
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/lgpl.html
 */
package jahuwaldt.js.ParamTools;

import jahuwaldt.swing.undo.ExtendedUndoManager;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.net.MalformedURLException;
import java.text.MessageFormat;
import java.util.Iterator;
import java.util.List;
import java.util.ResourceBundle;
import javax.swing.JPanel;


/**
 * A panel that displays the input boxes for a set of parameters in a parameter
 * database.
 * 
 * <p> Modified by: Joseph A. Huwaldt   </p>
 * 
 * @author Joseph A. Huwaldt Date: November 23, 1997
 * @version June 15, 2015
 */
public class ParamDBInputPanel extends JPanel {

	private static final long serialVersionUID = 3988173152776697237L;

    /**
     * Construct a new parameter database input panel.
     *
     * @param pdb The parameter database to display the input boxes for.
     * @param pLabels Parameter labels indicating the order the parameter should
     * appear in the display. If <code>null</code> is passed, then the
     * parameters are displayed in the order that they appear in the parameter
     * database.
     * @param columns The number of columns to use when displaying the input boxes.
     * @param resBundle The resource bundle for this application's strings. Must
     *                  include "paramNotFoundMsg".
     * @param undo Support for undo/redo. If <code>null</code> is passed, then
     *                  there will be no undo/redo support.
     * @throws java.net.MalformedURLException
     */
	public ParamDBInputPanel(ParamDatabase pdb, List<String> pLabels, int columns, ResourceBundle resBundle,
								ExtendedUndoManager undo) throws MalformedURLException {
		super(new GridBagLayout());
		GridBagLayout gridbag = (GridBagLayout)this.getLayout();
		
		int xPos=0, yPos=0;

		//	Set up for grid layout.
		GridBagConstraints constraints = new GridBagConstraints();
		
		//	If we don't have parameter labels indicating order, get an iterator that
		//	will deliver parameters in an arbitrary order.
		Iterator<Param<?>> iterator = null;
		if (pLabels == null) 
			iterator = pdb.iterator();

		//	Loop over elements in database, placing each one in this container.
		int size = pdb.size();
		for (int i=0; i < size; ++i) {
			buildConstraints(constraints, xPos,yPos, 1,1, 100/columns, 10);
			constraints.fill = GridBagConstraints.BOTH;		//	How the component should fill cell.
			constraints.anchor = GridBagConstraints.CENTER;	//	Justification in cell
			
			//	Get the next element from database.
			AutoParam<?> theParam;
			String label;
			if (pLabels != null) {
				label = pLabels.get(i);
				theParam = (AutoParam<?>)pdb.get(label);
				
			} else {
				theParam = (AutoParam<?>)iterator.next();
				label = theParam.getLabel();
			}
			
			if (theParam == null) {
                String msg = MessageFormat.format(resBundle.getString("paramNotFoundMsg"),label);
				throw new NullPointerException(msg);
            }
            
			//	Create text field for inputing this parameter's value.
			AutoParamInput thePI = new AutoParamInput( theParam );
			if (undo != null)	thePI.addUndoableEditListener(undo);
			gridbag.setConstraints(thePI, constraints);
			this.add(thePI);
			
			++xPos;
			if (xPos >= columns) {
				xPos = 0;
				++yPos;
			}
		}
		
	}
	
    /**
     * Fills in the GridBagConstraints record for a given component with input
     * items.
     *
     * @param gx,gy Grid (cell) indices for this component in the x and y dir
     *              (1,3 == 2nd column, 4th row).
     * @param gw,gh The number of cells this component spans.
     * @param wx,wy Proportional width/height of this grid cell compared to others.
     */
	private static void buildConstraints(GridBagConstraints gbc, int gx, int gy,
								int gw, int gh, int wx, int wy) {
		gbc.gridx = gx;
		gbc.gridy = gy;
		
		gbc.gridwidth = gw;
		gbc.gridheight = gh;
		
		gbc.weightx = wx;
		gbc.weighty = wy;
		
	}
	
}
