/*
*   ParamMethod - Interface for objects that contain a method used to calculate an AutoParam.
*
*   Copyright (C) 1998-2012 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.ParamTools;

import jahuwaldt.js.param.Parameter;
import javax.measure.quantity.Quantity;


/**
*  <p>  The interface for various parameter method objects which contain a 
*       method for calculating a given parameter's value from a given list
*       of input parameter values.
*  </p>
*
*  <p>  Modified by:  Joseph A. Huwaldt  </p>
*
*  @author   Joseph A. Huwaldt   Date: February 1, 1998
*  @version  May 27, 2012
**/
public interface ParamMethod<Q extends Quantity> {

	/**
	*  The user of this class should supply a routine that calculates
	*  the parameter value with a given set of input values.  If
	*  an exception occurs that can't be handled in calc(), it may be
	*  thrown.  It will be intercepted and reported back to the user
	*  by the {@link AutoParamInput} class.
	**/
	@SuppressWarnings("rawtypes")
	public Parameter<Q> calc( Parameter[] inputs ) throws Exception;

	/**
	*  Returns true if the inputs for this parameter are appropriate
	*  for this method.  Normally, "valid" means that the values for the input
	*  parameters are within a certain range.
	**/
	public boolean validInputs(ParamDatabase pdb, String[] inputs);
	
	/**
	*  Returns the number of inputs required by this method (the
	*  number of input parameter labels passed to the constructor when
	*  this object was instantiated).
	*
	*  @return  Returns the number of input values required by this method.
	**/
	public int getNumInputs();

	/**
	*  Returns the label of the input parameter specified by "index".
	*
	*  @param   index  Index into the input label array for the label to be returned.
	*  @return  Returns the label for the input parameter specified.
	*  @exception  ArrayIndexOutOfBoundsException  Thrown if index is < 0 or greater
	*              than the number of parameters available.
	**/
	public String getInputLabel( int index ) throws ArrayIndexOutOfBoundsException;

	/**
	*  Returns the array of all the input parameter labels.
	*
	*  @return  The array of all input parameter label strings is returned.
	**/
	public String[] getAllLabels();

}


