/*
*   StdAtmos -- Abstract class providing interface for standard atmosphere models.
*   
*   Copyright (C) 1998-2012 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Lesser General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*   Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.aero;

import javax.measure.quantity.Length;
import javax.measure.quantity.Velocity;
import javax.measure.quantity.Pressure;
import javax.measure.quantity.Temperature;
import javax.measure.quantity.VolumetricDensity;
import javax.measure.quantity.Dimensionless;

import javolution.lang.Realtime;
import javolution.xml.XMLSerializable;
import javolution.text.Text;
import javolution.text.TextBuilder;

import jahuwaldt.js.param.Parameter;


/**
*  An abstract class that provides standard functionality
*  for all atmosphere models that sub-class off of this
*  one.  Sub-classes must provide a method for calculating
*  the atmosphere properties of density ratio, temperature
*  ratio and pressure ratio.  Also methods must be provided
*  that simply return the values of temperature, pressure,
*  density and speed of sound at sea level conditions.
*
*  <p>  Modified by:  Joseph A. Huwaldt  </p>
*
*  @author  Joseph A. Huwaldt  Date:	September 27, 1998
*  @version May 22, 2012
**/
@SuppressWarnings("serial")
public abstract class StdAtmos implements Realtime, XMLSerializable {

	/**
	* The current geometric altitude in meters.
	*/
	protected  Parameter<Length> _alt = Parameter.ZERO_LENGTH;

	/**
	* The density at altitude / sea-level standard density.
	*/
	protected  Parameter<Dimensionless> _sigma = Parameter.ONE;

	/**
	* The pressure at altitude / sea-level standard pressure.
	*/
	protected  Parameter<Dimensionless> _delta = Parameter.ONE;

	/**
	* The temp. at altitude / sea-level standard temperature.
	*/
	protected  Parameter<Dimensionless> _theta = Parameter.ONE;

	/**
	* The speed-of-sound at altitude / sea-level standard speed-of-sound.
	*/
	protected  Parameter<Dimensionless> _cs = Parameter.ONE;
    
    
	//-----------------------------------------------------------------------------------
	/**
	*  Get geometric altitude currently being used for standard
	*  atmosphere calculations.
	*
	*  @return Returns altitude where the standard atmosphere is
	*          being calculated.
	**/
	public final Parameter<Length> getAltitude() {
		return _alt;
	}

	/**
	*  Get the density at altitude divided by the sea-level
	*  standard density.
	*
	*  @return Returns the density ratio at altitude.
	**/
	public final Parameter<Dimensionless> getDensityRatio() {
		return _sigma;
	}

	/**
	*  Get the pressure at altitude divided by the sea-level
	*  standard pressure.
	*
	*  @return Returns the pressure ratio at altitude.
	**/
	public final Parameter<Dimensionless> getPressureRatio() {
		return _delta;
	}

	/**
	*  Get the temperature at altitude divided by the sea-level
	*  standard temperature.
	*
	*  @return Returns the temperature ratio at altitude.
	**/
	public final Parameter<Dimensionless> getTemperatureRatio() {
		return _theta;
	}

	/**
	 *  Get the speed of sound at altitude divided by the sea-level
	 *  standard speed of sound.
	 *
	 *  @return Returns the speed of sound ratio at altitude.
	 **/
	public final Parameter<Dimensionless> getSpeedOfSoundRatio() {
		return _cs;
	}
	
	/**
	*  Get the static air temperature at altitude.
	*
	*  @return Returns the static air temperature at altitude.
	**/
	@SuppressWarnings("unchecked")
	public final Parameter<Temperature> getTemperature() {
		return T0().times(_theta).asType(Temperature.class);
	}

	/**
	*  Get the static air pressure at altitude.
	*
	*  @return Returns the pressure at altitude.
	**/
	@SuppressWarnings("unchecked")
	public final Parameter<Pressure> getPressure() {
		return P0().times(_delta).asType(Pressure.class);
	}

	/**
	*  Get the static air density at altitude.
	*
	*  @return Returns the density at altitude.
	**/
	@SuppressWarnings("unchecked")
	public final Parameter<VolumetricDensity> getDensity() {
		return RHO0().times(_sigma).asType(VolumetricDensity.class);
	}

	/**
	*  Get the speed of sound at altitude.
	*
	*  @return Returns the speed of sound at altitude.
	**/
	@SuppressWarnings("unchecked")
	public final Parameter<Velocity> getSpeedOfSound() {
		return a0().times( _cs ).asType(Velocity.class);
	}
	
	/**
 	* Compares this standard atmosphere model against the specified object for strict 
 	* equality (same model type, same altitude).
	*
	*  @param obj  The object we are comparing this one to.
	*  @return <code>true</code> if the given object is equal to this one.
	**/
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if ((obj == null) || (obj.getClass() != this.getClass()))
			return false;
		
		StdAtmos that = (StdAtmos)obj;
		
		if (!this._alt.equals(that._alt))		return false;
		
		return true;
	}

    /**
    *  Returns a hash code value for the object.
    **/
    public int hashCode() {
		int hash = 7;
		
		int var_code = 0;
		var_code = _alt.hashCode();
		hash = hash*31 + var_code;
		
		return hash;
	}
	
 	/**
 	* Returns the text representation of this object.
 	*
	* @return the text representation of this object.
 	*/
	public Text toText() {
		TextBuilder tmp = TextBuilder.newInstance();
		tmp.append("{ ");
		tmp.append("altitude = ");
		tmp.append( _alt.toText() );
		tmp.append(", DensityR = ");
		tmp.append( _sigma.toText() );
		tmp.append(", PressureR = ");
		tmp.append( _delta.toText() );
		tmp.append(", TemperatureR = ");
		tmp.append( _theta.toText() );
		tmp.append(", aR = ");
		tmp.append( _cs.toText() );
		tmp.append(" }");
		Text txt = tmp.toText();
		TextBuilder.recycle(tmp);
		return txt;
	}
	
	/**
 	* Returns the text representation of this object as a 
 	* <code>java.lang.String</code>.
 	* 
 	* @return <code>toText().toString()</code>
 	*/
	public final String toString() {
		return toText().toString();
	}
	
	
	//-----------------------------------------------------------------------------------
	/**
	*  Returns the standard sea level temperature for this
	*  atmosphere model.  Value returned in Kelvins.
	*
	*  @return Returns the standard sea level temperature.
	**/
	public abstract Parameter<Temperature> T0();

	/**
	*  Returns the standard sea level pressure for this
	*  atmosphere model.  Value returned in Newtons/m^2.
	*
	*  @return Returns the standard sea level pressure.
	**/
	public abstract Parameter<Pressure> P0();

	/**
	*  Returns the standard sea level density for this
	*  atmosphere model.  Value returned in kg/L (g/cm^3).
	*
	*  @return Returns the standard sea level density.
	**/
	public abstract Parameter<VolumetricDensity> RHO0();

	/**
	*  Returns the standard sea level speed of sound for this
	*  atmosphere model.  Value returned in meters/sec.
	*
	*  @return Returns the standard sea level speed of sound.
	**/
	public abstract Parameter<Velocity> a0();

	/**
	*  Returns the minimum altitude supported by this
	*  atmosphere model.  Sub-classes should return the
	*  minimum altitude supported the the sub-class'
	*  atmosphere model.
	*
	*  @return Returns the minimum altitude supported by this
	*          atmosphere model.
	**/
	public abstract Parameter<Length> minAltitude();

	/**
	*  Returns the maximum altitude supported by this
	*  atmosphere model.  Sub-classes should return the
	*  maximum altitude supported the the sub-class'
	*  atmosphere model.
	*
	*  @return Returns the maximum altitude supported by this
	*          atmosphere model.
	**/
	public abstract Parameter<Length> maxAltitude();

	/**
	*  Sets the geometric altitude where the standard
	*  atmosphere is to be calculated.
	*
	*  @param  altitude Geometric altitude at which standard atmosphere is
	*          to be calculated.
	**/
	public abstract void setAltitude( Parameter<Length> altitude ) throws IllegalArgumentException;

}


