/*
*   DensityVerifier -- Verifies that density inputs are reasonable.
*   
*   Copyright (C) 1999 - 2012 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.pmethods.stdatmos;

import java.awt.Component;
import javax.swing.*;

import javax.measure.quantity.VolumetricDensity;
import javax.measure.quantity.Length;
import javax.measure.unit.SI;
import javax.measure.unit.NonSI;

import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.ParamTools.*;


/**
*  Verifies that the density the user entered is less than
*  RHO0 and greater than 6.95553e-09 g/cm^3.
*
*  <p> Modified by:  Joseph A. Huwaldt     </p>
*
*  @author   Joseph A. Huwaldt  Date: October 13, 1998
*  @version  February 18, 2012
**/
public class DensityVerifier implements ParamVerifier, AtmosSharedData {
	
	private final Parameter<VolumetricDensity> MINRHO0;
	
	public DensityVerifier() {
		//	Ask the model what the minimum value is.
		Parameter<Length> oldAlt = atmosModel.getAltitude();
		atmosModel.setAltitude(atmosModel.maxAltitude());
		MINRHO0 = atmosModel.getDensity();
		atmosModel.setAltitude(oldAlt);
	}
	
	/**
	*  Check to see if the input value is valid for this parameter.	</p>
	*
	*  @param   root   The root component containing the parameter input text field
	*                  for the parameter being verified.
	*  @param   value  The value of the parameter as input by the user.
	*  @return  If "value" is valid, it should be output.  If "value" is invalid or
	*           out of range, a valid value is output or Float.NaN if nothing else.
	*           If ParamInput (which calls this routine) recieves a value of Float.NaN,
	*           it will clear the parameter.
	**/
	public Parameter check(Component root, Parameter value) {
		Parameter output = value;
		
		if (output.isGreaterThan(atmosModel.RHO0())) {
			output = atmosModel.RHO0().to(value.getUnit());

			JOptionPane.showMessageDialog ( root,
									"Density can not be greater than " + output + ".",
									"Out of Range",
									JOptionPane.ERROR_MESSAGE );

		} else if (output.isLessThan(MINRHO0)) {
			output = MINRHO0;

			JOptionPane.showMessageDialog ( root,
									"Density can not be less than " + output.to(value.getUnit()) + ".",
									"Out of Range",
									JOptionPane.ERROR_MESSAGE );
		}
		
		return output;
	}

}
