/*
*   Mfn_TT_T -- Calculates Mach number as a function of TT and T.
*   
*   Copyright (C) 1999 - 2011 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.pmethods.stdatmos;

import javax.measure.quantity.Dimensionless;
import javax.measure.quantity.Temperature;

import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.ParamTools.*;


/**
*  Calculates the Mach number based on total temperature
*  and static temperature.
*
*  <p>  Modified by:  Joseph A. Huwaldt   </p>
*
*  @author   Joseph A. Huwaldt  Date: October 18, 1998
*  @version  July 23, 2011
**/
public class Mfn_TT_T extends AbstractParamMethod<Dimensionless> implements AtmosSharedData {
	
	/**
	*  Constructor for this method.
	*
	*  @param   TT  Label for the "total temperature" parameter.
	*  @param   T   Label for the "static temperature" parameter.
	**/
	public Mfn_TT_T(String TT, String T) {
		_inputLabels = new String[2];
		_inputLabels[0] = TT;
		_inputLabels[1] = T;
	}
	
	
	/**
	*  Calculates the Mach number based on total temperature and static temperature.
	*  Method:  M = sqrt(2/(gam-1)*( (TT/T) - 1))
	*
	*  @param   inputs  Array of input parameters.
	*                   [0=total temperature (TT), 1=static temperature at altitude (T)].
	*  @return  Mach number (M).
	**/
	public Parameter<Dimensionless> calc(Parameter[] inputs) {
		double TT = inputs[0].doubleValue(Temperature.UNIT);
		double T = inputs[1].doubleValue(Temperature.UNIT);
		double	gam = GAMMA;
		
		double M = Math.sqrt((2./(gam-1.)*(TT/T - 1.)));
		
		return Parameter.valueOf(M, Dimensionless.UNIT);
	}

}
