/*
*   Mfn_rhoT_rho -- Calculates Mach number as a function of rhoT and rho.
*   
*   Copyright (C) 1999 - 2011 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.pmethods.stdatmos;

import javax.measure.quantity.Dimensionless;
import javax.measure.quantity.VolumetricDensity;

import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.ParamTools.*;


/**
*  Calculates the Mach number based on total density
*  and static density.
*
*  <p>  Modified by:  Joseph A. Huwaldt   </p>
*
*  @author   Joseph A. Huwaldt  Date: October 18, 1998
*  @version  July 24, 2011
**/
public class Mfn_rhoT_rho extends AbstractParamMethod<Dimensionless> implements AtmosSharedData {
	
	/**
	*  Constructor for this method.
	*
	*  @param   rhoT  Label for the "total density" parameter.
	*  @param   rho   Label for the "static density" parameter.
	**/
	public Mfn_rhoT_rho(String rhoT, String rho) {
		_inputLabels = new String[2];
		_inputLabels[0] = rhoT;
		_inputLabels[1] = rho;
	}
	
	
	/**
	*  Calculates the Mach number based on total density and static density.
	*  Method:  M = sqrt(2/(gam-1)*( (rhoT/rho)^(gam-1) - 1))
	*
	*  @param   inputs  Array of input parameters.
	*                   [0=total density (rhoT), 1=static density at altitude (rho)].
	*  @return  Mach number (M).
	**/
	public Parameter<Dimensionless> calc(Parameter[] inputs) {
		double rhoT = inputs[0].doubleValue(VolumetricDensity.UNIT);
		double rho = inputs[1].doubleValue(VolumetricDensity.UNIT);
		double	gam1 = GAMMA - 1;
		
		double M = Math.sqrt((2./gam1*(Math.pow(rhoT/rho, gam1) - 1.)));
		
		return Parameter.valueOf(M, Dimensionless.UNIT);
	}

}
