/*
*   PTfn_M_P -- Calculates total pressure as a function of M and P.
*   
*   Copyright (C) 1999 - 2011 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.pmethods.stdatmos;

import javax.measure.quantity.Dimensionless;
import javax.measure.quantity.Pressure;

import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.ParamTools.*;


/**
*  Calculates total pressure based on Mach number and
*  static pressure at alititude.
*
*  <p>  Modified by:  Joseph A. Huwaldt    </p>
*
*  @author  Joseph A. Huwaldt   Date: October 18, 1998
*  @version July 6, 2012
**/
public class PTfn_M_P extends AbstractParamMethod<Pressure> implements AtmosSharedData {
	
	/**
	*  Constructor for this method.
	*
	*  @param   M	Label for the "Mach number" parameter.
	*  @param   P	Label for the "static pressure" parameter.
	**/
	public PTfn_M_P(String M, String P) {
		_inputLabels = new String[2];
		_inputLabels[0] = M;
		_inputLabels[1] = P;
	}
	
	
	/**
	*  Calculates total pressure based on Mach number and static pressure at alititude.
	*  Method:  PT = P*(1 + (gam-1)/2*M^2)^( gam/(gam-1) )
	*
	*  @param   inputs  Array of input parameters.
	*                   [0=Mach number (M), 1=static pressure at altitude (P)].
	*  @return  Total pressure (PT).
	**/
	public Parameter<Pressure> calc(Parameter[] inputs) {
		double M = inputs[0].doubleValue(Dimensionless.UNIT);
		double P = inputs[1].getValue();
		double gam = GAMMA;
		
		double temp = 1. + 0.5*(gam-1.)*M*M;
		temp = Math.pow(temp, gam/(gam-1.));
		
		double PT = P*temp;
		
		return Parameter.valueOf(PT, inputs[1].getUnit());
	}

}


