/*
*   Tfn_M_TT -- Calculates static temperature as a function of M & TT.
*   
*   Copyright (C) 1999 - 2011 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.pmethods.stdatmos;

import javax.measure.quantity.Dimensionless;
import javax.measure.quantity.Temperature;

import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.ParamTools.*;


/**
*  Calculates static temperature at altitude based on
*  Mach number and total temp.
*
*  <p>  Modified by:  Joseph A. Huwaldt    </p>
*
*  @author  Joseph A. Huwaldt   Date: January 29, 1999
*  @version July 24, 2011
**/
public class Tfn_M_TT extends AbstractParamMethod<Temperature> implements AtmosSharedData {
	
	/**
	*  Constructor for this method.
	*
	*  @param   M   Label for the "Mach number" parameter.
	*  @param   TT  Label for the "total temperature" parameter.
	**/
	public Tfn_M_TT(String M, String TT) {
		_inputLabels = new String[2];
		_inputLabels[0] = M;
		_inputLabels[1] = TT;
	}
	
	
	/**
	*  Calculates static temperature at altitude based on Mach number and total temp.
	*  Method:  T = TT/(1 + (gam-1)/2*M^2)
	*
	*  @param   inputs  Array of input parameters.
	*                   [0=Mach number (M), 1=total temperature (TT)].
	*  @return  Static temperature at altitude (T) in Kelvin.
	**/
	public Parameter<Temperature> calc(Parameter[] inputs) {
		double M = inputs[0].doubleValue(Dimensionless.UNIT);
		Parameter TT = inputs[1].to(Temperature.UNIT);
		
		Parameter T = TT.divide(1 + 0.5*(GAMMA-1)*M*M);
		
		return T;
	}

}


