/*
*   VCfn_PT_P -- Calculates calibrated airspeed as a function of PT and P.
*   
*   Copyright (C) 1998 - 2011 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.pmethods.stdatmos;

import javax.measure.quantity.Velocity;
import javax.measure.quantity.Pressure;

import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.ParamTools.*;


/**
*  Calculates calibrated airspeed based on total pressure
*  and static pressure.
*
*  <p>  Modified by:  Joseph A. Huwaldt    </p>
*
*	@author		Joseph A. Huwaldt	Date:	October 18, 1998
*	@version	July 24, 2011
**/
public class VCfn_PT_P extends AbstractParamMethod<Velocity> implements AtmosSharedData {
	
	/**
	*  Constructor for this method.
	*
	*  @param   PT  Label for the "total pressure" parameter.
	*  @param   P   Label for the "static pressure" parameter.
	**/
	public VCfn_PT_P(String PT, String P) {
		_inputLabels = new String[2];
		_inputLabels[0] = PT;
		_inputLabels[1] = P;
	}
	
	
	/**
	*  Calculates calibrated airspeed based on total pressure and static pressure.
	*  Method:  VC = sqrt( 2*a0^2/(gam-1)*(((PT-P)/P0 + 1)^((gam-1)/gam) - 1) )
	*
	*  @param   inputs  Array of input parameters.
	*                   [0=total pressure (PT), 1=static pressure at altitude (P)].
	*  @return  Calibrated airspeed (VC) in m/s.
	**/
	public Parameter<Velocity> calc(Parameter[] inputs) {
		double PT = inputs[0].doubleValue(Pressure.UNIT);
		double P = inputs[1].doubleValue(Pressure.UNIT);
		double a0 = atmosModel.a0().doubleValue(Velocity.UNIT);
		double P0 = atmosModel.P0().doubleValue(Pressure.UNIT);
		
		double temp = (PT - P)/P0 + 1;
		temp = Math.pow(temp, (GAMMA-1)/GAMMA);
		
		double VC = Math.sqrt( 2*a0*a0/(GAMMA-1)*(temp - 1) );
		
		return Parameter.valueOf(VC, Velocity.UNIT);
	}

}


