/*
*   VTfn_VE_sigma -- Calculates true airspeed as a function of VE and sigma.
*   
*   Copyright (C) 1999 - 2011 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.pmethods.stdatmos;

import javax.measure.quantity.Velocity;
import javax.measure.quantity.Dimensionless;

import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.ParamTools.*;


/**
*  Calculates the true airspeed based on equivalent airspeed
*  and density ratio at altitude.
*
*  <p>  Modified by:  Joseph A. Huwaldt  </p>
*
*  @author   Joseph A. Huwaldt   Date:  October 18, 1998
*  @version  July 23, 2011
**/
public class VTfn_VE_sigma extends AbstractParamMethod<Velocity> {
	
	/**
	*  Constructor for this method.
	*
	*  @param   VE     Label for the "equivalent airspeed" parameter.
	*  @param   sigma  Label for the "static density ratio" parameter.
	**/
	public VTfn_VE_sigma(String VE, String sigma) {
		_inputLabels = new String[2];
		_inputLabels[0] = VE;
		_inputLabels[1] = sigma;
	}
	
	
	/**
	*  Calculates the true airspeed based on equivalent airspeed
	*  and density ratio at altitude.
	*  Method:  VT = VE/sqrt(sigma)
	*
	*  @param   inputs  Array of input parameters.
	*                   [0=equivalent airspeed (VE),
	*                    1=density ratio with altitude (sigma)].
	*  @return  True airspeed (VT) in m/s.
	**/
	public Parameter<Velocity> calc(Parameter[] inputs) {
		Parameter<Velocity> VE = inputs[0];
		Parameter<Dimensionless> sigma = inputs[1];
		
		Parameter VT = VE.divide(sigma.sqrt());
		
		return VT;
	}

}


