/*
*   hfn_VE_M -- Calculates altitude as a function of VE and M.
*   
*   Copyright (C) 1998 - 2024 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.pmethods.stdatmos;

import javax.measure.quantity.Dimensionless;
import javax.measure.quantity.Length;
import javax.measure.quantity.Velocity;
import javax.measure.unit.SI;

import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.ParamTools.*;
import jahuwaldt.tools.math.Roots;
import jahuwaldt.tools.math.AbstractEvaluatable1D;


/**
*  Calculates the geometric altitude based on equivalent
*  airspeed and Mach number.
*
*  <p>  Modified by:  Joseph A. Huwaldt   </p>
*
*  @author   Joseph A. Huwaldt  Date: October 24, 1998
*  @version  April 13, 2024
**/
public class hfn_VE_M extends AbstractParamMethod<Length> implements AtmosSharedData {
	private static final DeltaSigmaTheta deltaSigTheta = new DeltaSigmaTheta();
	
	/**
	*  Constructor for this method.
	*
	*  @param  VE  Label for the "equivalent airspeed" parameter.
	*  @param  M   Label for the "Mach number" parameter.
	**/
	public hfn_VE_M(String VE, String M) {
		_inputLabels = new String[2];
		_inputLabels[0] = VE;
		_inputLabels[1] = M;
	}
	
	
	/**
	*  Calculates the geometric altitude based on equivalent airspeed and Mach number.
	*  Method:  Iterates on altitude until the given sigma*theta = (VE/M/a0)^2 factor
	*  is calculated.
	*
	*  @param   inputs  Array of input parameters.
	*                   [0=equivalent airspeed (VE), 1=Mach number (M)].
	*  @return  Geometric altitude in base units of length.
	*  @throws  RootException if there is a problem with the root solver.
	**/
    @Override
	public Parameter<Length> calc(Parameter[] inputs) throws Exception {
		double VE = inputs[0].doubleValue(Velocity.UNIT);
		double M = inputs[1].doubleValue(Dimensionless.UNIT);
		double h = atmosModel.maxAltitude().doubleValue(SI.METER);
		double a0 = atmosModel.a0().doubleValue(Velocity.UNIT);
		
		double sigTheta0 = VE/M/a0;
		sigTheta0 *= sigTheta0;		//	sigma*theta = (VE/M/a0)^2
		
		if (sigTheta0 > 2.90694e-15*3.47041 ) {
			deltaSigTheta.setInitialValue(sigTheta0);	//	The target value we're trying to find.
			double hMin = atmosModel.minAltitude().doubleValue(SI.METER);
			h = Roots.findRoot1D( deltaSigTheta, hMin, h, 0.0001);
		}
			
		return Parameter.valueOf(h, Length.UNIT);
	}

    /**
    *  Calculates the delta sigma-theta factor based on geometric altitude.
    *  This classes methods are called from the altitude root-finder.
    **/
    private static class DeltaSigmaTheta extends AbstractEvaluatable1D implements AtmosSharedData {
	
        /**
        *  Calculates the difference between the initial sigma*theta
        *  and sigma*theta at a particular altitude (target sigma-theta factor).
        *
        *  @param   h  Geometric altitude in meters.
        *  @return  The sigma-theta factor at altitude - target sigma-theta factor.
        **/
        @Override
        public double function(double h) {
            atmosModel.setAltitude(Parameter.valueOf(h, Length.UNIT));		//	Set altitude in meters.
            double sigma = atmosModel.getDensityRatio().getValue();
            double theta = atmosModel.getTemperatureRatio().getValue();
            double del = sigma*theta - getInitialValue();
		
            return del;
        }
    }

}

