/*
*   hfn_VE_VC -- Calculates altitude as a function of VE and VC.
*   
*   Copyright (C) 1998 - 2012 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.pmethods.stdatmos;

import javax.measure.quantity.Dimensionless;
import javax.measure.quantity.Length;
import javax.measure.quantity.Velocity;
import javax.measure.unit.SI;

import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.ParamTools.*;
import jahuwaldt.tools.math.Roots;
import jahuwaldt.tools.math.AbstractEvaluatable1D;


/**
*  Calculates the geometric altitude based on equivelent airspeed & calibrated airspeed.
*
*  @author   Joseph A. Huwaldt   Date:  October 28, 1998
*  @version  February 18, 2012
**/
public class hfn_VE_VC extends AbstractParamMethod<Length> implements AtmosSharedData {
	private static final DeltaDeltafnVEVC deltaDelta = new DeltaDeltafnVEVC();
	
	/**
	*  Constructor for this method.
	*
	*  @param   VE  Label for the "equivalent airspeed" parameter.
	*  @param   VC  Label for the "calibrated airspeed" parameter.
	**/
	public hfn_VE_VC(String VE, String VC) {
		_inputLabels = new String[2];
		_inputLabels[0] = VE;
		_inputLabels[1] = VC;
	}
	
	
	/**
	*  Calculates the geometric altitude based on equivelent airspeed & calibrated airspeed.
	*  Method:  Iterates on altitude until the given function delta = fn(VE, VC, delta)
	*  is calculated.
	*
	*  @param   inputs  Array of input parameters.
	*                   [0=equivalent airspeed (VE), 1=calibrated airspeed (VC)].
	*  @return  Geometric altitude in base units of length.
	*  @throws  RootException if there is a problem with the root solver.
	**/
	public Parameter<Length> calc(Parameter[] inputs) throws Exception {
		double VE = inputs[0].doubleValue(Velocity.UNIT);
		double VC = inputs[1].doubleValue(Velocity.UNIT);
		double hMin = atmosModel.minAltitude().doubleValue(SI.METER);
		double hMax = atmosModel.maxAltitude().doubleValue(SI.METER);
		
		deltaDelta.VE = VE;
		deltaDelta.VC = VC;
		double h = Roots.findRoot1D( deltaDelta, hMin, hMax, 0.1);
		
		return Parameter.valueOf(h, Length.UNIT);
	}

    /**
    *  Calculates the delta "delta funtion" based on a guess
    *  at altitude.  This class' methods are called from the
    *  altitude root-finder.
    **/
    private static class DeltaDeltafnVEVC extends AbstractEvaluatable1D implements AtmosSharedData {
        double	VE;
        double	VC;
	
        /**
        *  Calculates the difference between the initial delta function
        *  and the delta function at a particular delta.
        *
        *  @param   h  Guess at altitude that satisfies delta equation.
        *  @return  The delta function at delta1 - delta1.
        **/
        public double function(double h) {
	
            double	gamR = GAMMA/(GAMMA - 1);
            double	a02 = atmosModel.a0().doubleValue(Velocity.UNIT);
            a02 *= a02;
            a02 = 0.5/a02;							//	= 1/(2*a0^2)
		
            atmosModel.setAltitude(Parameter.valueOf(h, Length.UNIT));
            double delta1 = atmosModel.getPressureRatio().getValue();
		
            double temp = 1 + (GAMMA - 1)*a02*VC*VC;
            temp = ( Math.pow(temp, gamR) - 1)/delta1 + 1;
            temp = Math.pow(temp, 1/gamR) - 1;
		
            double delta2 = VE*VE*(GAMMA-1)*a02/temp;
		
            double del = delta2 - delta1;
		
            return del;
        }
	
    }

}


