/*
*   hfn_theta -- Calculates altitude as a function of theta.
*   
*   Copyright (C) 1998 - 2012 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.pmethods.stdatmos;

import javax.measure.quantity.Dimensionless;
import javax.measure.quantity.Length;

import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.ParamTools.*;
import jahuwaldt.tools.math.Roots;
import jahuwaldt.tools.math.AbstractEvaluatable1D;


/**
*  <p>  Calculates the geometric altitude based on the temperature ratio.
*       There are generally two solutions to altitude for a given temperature
*       ratio.  The lowest altitude solution is always returned by this
*       routine.
*  </p>
*
*  <p>  Modified by:  Joseph A. Huwaldt  </p>
*
*  @author   Joseph A. Huwaldt   Date:  September 29, 1998
*  @version  February 18, 2012
**/
public class hfn_theta extends AbstractParamMethod<Length> implements AtmosSharedData {
	private static final DeltaThetaEval deltaTheta = new DeltaThetaEval();
	
	/**
	*  Constructor for this method.
	*
	*  @param  theta  Label for the "temperature ratio" parameter.
	**/
	public hfn_theta(String theta) {
		_inputLabels = new String[1];
		_inputLabels[0] = theta;
	}
	
	
	/**
	*  Calculates the geometric altitude based on the temperature ratio.
	*  Method:  Iterates on altitude until the given theta is calculated.
	*
	*  @param   inputs  Array of input parameters.
	*                   [0=temperature ratio (theta)].
	*  @return  Geometric altitude in base units of length.
	*  @throws  RootException if there is a problem with the root solver.
	**/
	public Parameter<Length> calc(Parameter[] inputs) throws Exception {
		double theta = inputs[0].doubleValue(Dimensionless.UNIT);
		double h = 11000;			//	Minimum temperature occurs at 11 km altitude.
		
		if ( theta > 0.75186534790908 ) {
			deltaTheta.setInitialValue(theta);	//	The target value we are trying to find.
			double hMin = atmosModel.minAltitude().doubleValue(Length.UNIT);
			h = Roots.findRoot1D( deltaTheta, hMin, 11000.0, 0.1);
		}
		
		return Parameter.valueOf(h, Length.UNIT);
	}

    /**
    *  Calculates the delta-temperature ratio based on geometric altitude.
    *  This class' methods are called from the altitude root-finder.
    **/
    private static class DeltaThetaEval extends AbstractEvaluatable1D implements AtmosSharedData {
	
        /**
        *  Calculates the difference between the initial theta
        *  and the theta at a particular altitude (target theta).
        *
        *  @param   h	Geometric altitude in meters.
        *  @return  The temperature ratio with altitude (theta) - target temperature ratio.
        **/
        public double function(double h) {
            atmosModel.setAltitude(Parameter.valueOf(h, Length.UNIT));		//	Set altitude in meters.
			double theta = atmosModel.getTemperatureRatio().getValue();
            double delta = theta - getInitialValue();
            return delta;
        }
	
    }

}


