/*
*   mufn_theta -- Calculates dynamic viscosity as a function of theta.
*   
*   Copyright (C) 1999 - 2011 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.pmethods.stdatmos;

import javax.measure.quantity.Dimensionless;
import javax.measure.quantity.DynamicViscosity;
import static javax.measure.unit.SI.*;
import static javax.measure.unit.NonSI.*;

import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.ParamTools.*;


/**
*  Calculates dynamic (absolute) viscosity as a function
*  of temperature ratio.
*
*  <p>  Modified by:  Joseph A. Huwaldt  </p>
*
*  @author   Joseph A. Huwaldt   Date:  January 31, 1999
*  @version  July 23, 2011
**/
public class mufn_theta extends AbstractParamMethod<DynamicViscosity> {
	
	/**
	*  Constructor for this method.
	*
	*  @param  theta  Label for the "temperature ratio" parameter.
	**/
	public mufn_theta(String theta) {
		_inputLabels = new String[1];
		_inputLabels[0] = theta;
	}
	
	
	/**
	*  Calculates dynamic (absolute) viscosity as a function of temperature ratio.
	*  Method:  mu = 5.3041E-7*theta^(3/2)/(theta + 0.41644)
	*
	*  @param   inputs  Array of input parameters.
	*                   [0=static temperature ratio with altitude (theta)].
	*  @return  Dynamic viscosity (mu) in poise.
	**/
	public Parameter<DynamicViscosity> calc(Parameter[] inputs) {
		double theta = inputs[0].doubleValue(Dimensionless.UNIT);
		
		double mu = 5.3041E-7*Math.pow(theta,1.5)/(theta + 0.41644);	//	Units = lbf-sec/ft^2
		
		//	Convert mu from lbf-sec/ft^2 to reference units
		Parameter mup = Parameter.valueOf(mu, POUND_FORCE.times(SECOND).divide(FOOT.pow(2)));
		
		return mup.to(DynamicViscosity.UNIT);
	}

}


