/*
*   rhofn_M_rhoT -- Calculates static density as a function of M and rhoT.
*   
*   Copyright (C) 1999 - 2011 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.pmethods.stdatmos;

import javax.measure.quantity.Dimensionless;
import javax.measure.quantity.VolumetricDensity;

import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.ParamTools.*;


/**
*  Calculates static density at altitude based on Mach
*  number and total density.
*
*  <p>  Modified by:  Joseph A. Huwaldt   </p>
*
*  @author   Joseph A. Huwaldt  Date: October 24, 1998
*  @version  July 23, 2011
**/
public class rhofn_M_rhoT extends AbstractParamMethod<VolumetricDensity> implements AtmosSharedData {
	
	/**
	*  Constructor for this method.
	*
	*  @param   M     Label for the "Mach number" parameter.
	*  @param   rhoT  Label for the "total density" parameter.
	**/
	public rhofn_M_rhoT(String M, String rhoT) {
		_inputLabels = new String[2];
		_inputLabels[0] = M;
		_inputLabels[1] = rhoT;
	}
	
	
	/**
	*  Calculates static density at altitude based on Mach number and total density.
	*  Method:  rho = rhoT/(1 + (gam-1)/2*M^2)^(1/(gam-1))
	*
	*  @param   inputs  Array of input parameters.
	*                   [0=Mach number (M), 1=total density (rhoT)].
	*  @return  Static density at altitude (rho) in kg/L.
	**/
	public Parameter<VolumetricDensity> calc(Parameter[] inputs) {
		double M = inputs[0].doubleValue(Dimensionless.UNIT);
		double rhoT = inputs[1].getValue();
		double gam = GAMMA;
		
		double temp = 1. + 0.5*(gam-1.)*M*M;
		temp = Math.pow(temp, 1./(gam-1.));
		
		double rho = rhoT/temp;
		
		return Parameter.valueOf(rho, inputs[1].getUnit());
	}

}


