/*
*   BrowserHelp - Provide "help" information to a program by opening an
*                 HTML document using a web browser.
*
*   Copyright (C) 1999-2023 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*   Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.util;

import java.awt.Desktop;
import java.net.URL;
import static java.util.Objects.nonNull;


/**
*  A class that provides "help" information to an application
*  or applet by opening a selected HTML page in a web browser.
*
*  <p>  Modified by:  Joseph A. Huwaldt    </p>
*
*  @author  Joseph A. Huwaldt   Date: October 7, 1999
*  @version December 5, 2023
**/
public class BrowserHelp implements HelpSystem, java.io.Serializable {

	/**
	*  URL to web page with "help/info" information.
	**/
	private URL helpURL = null;
		

	//-----------------------------------------------------------------------------
	/**
	*  Don't allow users access to the default constructor.
	**/
	protected BrowserHelp() { }

	/**
	*  Constructor that accepts the URL pointing to help information
	*  to show.  This constructor is intended for use in an application
	*  where the applet methods are not available.
	*
	*  @param  helpURL    The URL pointing to help information to show in a
	*                     web browser.
	**/
	public BrowserHelp( URL helpURL ) {
		this.helpURL = helpURL;
	}

	//-----------------------------------------------------------------------------
	/**
	*  Method that displays the stored URL in a web browser when called.
	**/
    @Override
	public void showHelp() {
	
        // If help is available, show the specified URL in a web browser.
        if (helpAvailable()) {
            // If in application, launch browser and show page.
            showDocument(helpURL);
		}
		
	}

	/**
	*  Returns true if help information is available, false if it is not.
	*
	*  @return Returns true if help information is available, false otherwise.
	**/
    @Override
	public boolean helpAvailable() {
		return (nonNull(helpURL));
	}

	/**
	*  Open the specified URL in the default web browser.
	**/
    private void showDocument( URL url ) {
        try {
            // Launch a web browser and display the requested URL.
            if (Desktop.isDesktopSupported() && Desktop.getDesktop().isSupported(Desktop.Action.BROWSE))
                Desktop.getDesktop().browse(url.toURI());
            else {
                System.out.println("Desktop.browse() is not supported on this platform.");
            }
        } catch (Exception err) {
            err.printStackTrace();
        }

	}
    

}


